/**
 * Implementazione di una coda "generica" mediante vettori.
 * 
 * @author Stefano Guerrini
 * @version maggio 2009
 */
public class ArrayCoda<T> implements Coda<T> {
    private int first;      // primo elemento
    private int end;        // indice della prima posizione libera
                            // alla fine della coda, dove inserire
                            // il prossimo elemento
    private T elements[];   // vettore per contenere gli elementi della coda
    static final int DEF_SIZE = 1024;   // dimensione di default
    
    /* Invarianti della classe
     * 
     * La coda contiene sempre 
     *      (end-first) mod DEF_SIZE
     * elementi.
     * 
     * Se la coda non e' vuota, first e' l'indice del primo elemento nella coda
     * last e' sempre l'indice di una posizione vuota dell'array, quella in cui
     * inserire il prossimo elemento da aggiungere alla coda (se la coda non e'
     * piena)
     * 
     * Se first < end gli elementi della coda di trovano nel segmento [first,end-1]
     * del vettorem ripsettanod l'ordine di inseirmento.
     * Se invece first > end,
     * allora gli elementi si trovano sel segmento [first,size-1] e [0,end-1], dove
     * size e' la dimensione della coda paria a elements.length.
     * Infine, se first = end, la coda e' vuota.
     * 
     * Se fist = end+1 oppure first = 0 e end = size-1 la pila e' piena, ovvero,
     * contiene size-1 elementi.
     */
    
    /**
     * Crea una coda
     * 
     * @param zise      la dimensione della coda
     *                  la coda puo' contenere al max size-1 elementi
     * @return          la nuova pila
     */
    public ArrayCoda(int size) {
        first = end = 0;
        elements = (T[]) new Object[size];
    }

    /**
     * Crea una coda di dimensione pari a DEF_SIZE
     * 
     * @return          la nuova coda
     */
    public ArrayCoda() {
        this(DEF_SIZE);
    }
    
    /**
     * Inserisce un elemento in cima alla coda
     * 
     * Pre:         la coda non deve essere piena
     * 
     * @param obj   l'elemento da inserire nella coda
     */
    public void enqueue(T obj) {
        elements[end++] = obj;
        if (end == elements.length)
            end = 0;
    }
    
    /**
     * Elimina l'elemento in cima alla coda
     * 
     * Pre:         la coda non deve essere vuota
     * 
     * @retun       l'elemento eliminato dalla coda
     */
    public T dequeue() {
        T elem = elements[first++];
        if (first == elements.length)
            first = 0;
        return elem;
    }
   
    /**
     * Prende l'elemento in cima alla pila,
     * senza toglierlo dalla pila
     * 
     * Pre:         la pila non deve essere vuota
     * 
     * @retun       l'elemento in cima alla pila
     */   
    public T first() {
        return elements[first];
    }
    
    /**
     * Verifica se la pila e' vuota
     * 
     * @retun       true se la pila e' vuota,
     *              false altrimenti
     */
    public boolean isEmpty() {
        return first == end;
    }

    /**
     * Verifica se la pila e' piena e non vi si possono
     * inserire altri elementi
     * 
     * @retun       true se la pila e' piena,
     *              false altrimenti
     */
    public boolean isFull() {
        return (numElem() == elements.length-1);
    }

    /**
     * Numero di elementi presenti nella pila
     * 
     * @retun       il numero di elementi nella pila
     */
    public int numElem() {
        if (end < first)
            return (end - first + elements.length);
        else
            return  (end - first);
        }

    /**
     * La dimensione della pila
     * 
     * @retun       la dimensione della pila se questa puo' contenere
     *              un numero massimo di elementi, o -1 se la pila
     *              puo' crescere senza limiti
     */
    public int size() {
        return elements.length;
    }
}
