/**
 * Implementazione di una pila "generica" mediante vettori.
 * 
 * @author Stefano Guerrini
 * @version maggio 2009
 */
public class ArrayPila<T> implements Pila<T> {
    private int numElem;    // numero di elementi nella pila
    private T elements[];   // vettore per contenere gli elementi della pila
    static final int DEF_SIZE = 1024;   // dimensione di default
    
    /**
     * Crea una pila
     * 
     * @param zise      la dimensione della pila
     * @return          la nuova pila
     */
    public ArrayPila(int size) {
        numElem = 0;
        elements = (T[]) new Object[size];
    }

    /**
     * Crea una pila di dimensione pari a DEF_SIZE
     * 
     * @return          la nuova pila
     */
    public ArrayPila() {
        this(DEF_SIZE);
    }
    
    /**
     * Inserisce un elemento in cima alla pila
     * 
     * Pre:         la pila non deve essere piena
     * 
     * @param obj   l'elemento da inserire nella pila
     */
    public void push(T obj) {
        elements[numElem++] = obj;
    }
    
    /**
     * Elimina l'elemento in cima alla pila
     * 
     * Pre:         la pila non deve essere vuota
     * 
     * @retun       l'elemento eliminato dalla pila
     */
    public T pop() {
        return elements[--numElem];
    }
   
    /**
     * Prende l'elemento in cima alla pila,
     * senza toglierlo dalla pila
     * 
     * Pre:         la pila non deve essere vuota
     * 
     * @retun       l'elemento in cima alla pila
     */   
    public T top() {
        return elements[numElem-1];
    }
    
    /**
     * Verifica se la pila e' vuota
     * 
     * @retun       true se la pila e' vuota,
     *              false altrimenti
     */
    public boolean isEmpty() {
        return numElem == 0;
    }

    /**
     * Verifica se la pila e' piena e non vi si possono
     * inserire altri elementi
     * 
     * @retun       true se la pila e' piena,
     *              false altrimenti
     */
    public boolean isFull() {
        return size() < numElem;
    }

    /**
     * Numero di elementi presenti nella pila
     * 
     * @retun       il numero di elementi nella pila
     */
    public int numElem() {
        return numElem;
    }

    /**
     * La dimensione della pila
     * 
     * @retun       la dimensione della pila se questa puo' contenere
     *              un numero massimo di elementi, o -1 se la pila
     *              puo' crescere senza limiti
     */
    public int size() {
        return elements.length;
    }
}
