
/**
 * Libreria di funzioni generiche da usare con le pile.
 * Mostra esempi di funzioni generiche
 * 
 * @author Stefano Guerrini
 * @version maggio 2009
 */
public class PilaLib{
    private PilaLib(){}
    
    /*
     * Versione static della push
     * Esempio di metodo generico
     * La variabile di tipo e' necessaria per leagre il tipo dell'elemento da
     * inserire a quello della pila in cui va inserito.
     * 
     * @param pila      la pila in cui inserire l'elemento
     * @param elem      l'elemento da inserire
     */
    public static <T> void push(Pila<T> pila, T elem) {
        pila.push(elem);
    }
    
    /*
     * Versione static della pop
     * Esempio di metodo generico
     * La variabile di tipo nella definizione della pila non e' necessaria.
     * Il tipo degli elementi della pila non e' utilizzato.
     * 
     * @param pila      la pila da cui eleiminare l'elemento
     */
    public static void pop1(Pila<?> pila) {
        pila.pop();
    }

    /*
     * Versione static della pop
     * Esempio di metodo generico
     * La variabile di tipo nella definizione e' necessaria per legare
     * il tipo dell'oggetto restituito dalla pila a quello degli
     * elementi nella pila.
     * 
     * I due metodi pop devono avere nomi diversi.
     * La compilazione dei metodi generici avviene per cancellazione dei tipi
     * generici, in molti punti rimpiazzati da Object. Per garantire i check
     * richiesti in aclni punti si insersicono anche dei downcast da Object.
     * pop1 e pop2, dopo la canzellazione dei generici hanno lo stesso tipo e
     * quindi devono avere nomi diversi
     * in pratica, i generici non possono essere usati per sovraccaricare i metodi
     * 
     * @param pila      la pila in cui eliminare l'elemento
     */
    public static <T> T pop2(Pila<T> pila) {
        return pila.pop();
    }

    /*
     * Svuota un pila
     * La pila e' generica.
     * Per specificare la classe non e' necessario utilizzare una variabile di tipo
     * ma si puo' usare una wildcard. Il tipo degli elementi della pila infatti non serve
     * nella scrittura del programma.
     * 
     * @param pila      la pila da svuotare
     */
    public static void svuota(Pila<?> pila) {
        while (!pila.isEmpty()) {
            pila.pop();
        }
    }

    /*
     * Sposta, rovesciandolo, il contenuto di una pila su quello di un'altra pila
     * Le due pile sono generiche.
     * Per come e' definita la signature del metodo, le due pile devono essere esattamente
     * dello stesso tipo.
     * 
     * @param from      la pila da svuotare
     * #param to        la pila destinazione
     */
    public static <T> void reverse1(Pila<T> from, Pila<T> to) {
        while (!from.isEmpty()) {
            to.push(from.pop());
        }
    }

    /*
     * Sposta, rovesciandolo, il contenuto di una pila su quello di un'altra pila
     * Le due pile sono generiche.
     * Si usa una wildcard per specificare che la pila da svuotare puo' anche essere di
     * istanze di un qualsiasi sotto-tipo della pila destinazione
     * 
     * @param from      la pila da svuotare
     * #param to        la pila destinazione
     */
    public static <T> void reverse2(Pila<? extends T> from, Pila<T> to) {
        while (!from.isEmpty()) {
            to.push(from.pop());
        }
    }

}
